<?php

/**
 * Chill is a software for social workers
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

declare(strict_types=1);

namespace Chill\MainBundle\Form;

use Chill\MainBundle\Entity\Workflow\EntityWorkflow;
use Chill\MainBundle\Entity\Workflow\EntityWorkflowStep;
use Chill\MainBundle\Form\Type\ChillTextareaType;
use Chill\MainBundle\Form\Type\PickUserDynamicType;
use Chill\MainBundle\Workflow\EntityWorkflowManager;
use LogicException;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Workflow\Registry;
use Symfony\Component\Workflow\Transition;

class WorkflowStepType extends AbstractType
{
    private EntityWorkflowManager $entityWorkflowManager;

    private Registry $registry;

    public function __construct(EntityWorkflowManager $entityWorkflowManager, Registry $registry)
    {
        $this->entityWorkflowManager = $entityWorkflowManager;
        $this->registry = $registry;
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        /** @var \Chill\MainBundle\Entity\Workflow\EntityWorkflow $entityWorkflow */
        $entityWorkflow = $options['entity_workflow'];
        $handler = $this->entityWorkflowManager->getHandler($entityWorkflow);

        if (true === $options['transition']) {
            if (null === $options['entity_workflow']) {
                throw new LogicException('if transition is true, entity_workflow should be defined');
            }

            $transitions = $this->registry
                ->get($options['entity_workflow'], $entityWorkflow->getWorkflowName())
                ->getEnabledTransitions($entityWorkflow);

            $choices = array_combine(
                array_map(static function (Transition $transition) { return $transition->getName(); }, $transitions),
                $transitions
            );

            $builder
                ->add('transition', ChoiceType::class, [
                    'label' => 'workflow.Transition',
                    'mapped' => false,
                    'multiple' => false,
                    'expanded' => true,
                    'choices' => $choices,
                    'choice_label' => static function (Transition $transition) {
                        return implode(', ', $transition->getTos());
                    },
                ])
                ->add('future_dest_users', PickUserDynamicType::class, [
                    'label' => 'workflow.dest for next steps',
                    'multiple' => true,
                    'mapped' => false,
                ]);
        }

        if (
            $handler->supportsFreeze($entityWorkflow)
            && !$entityWorkflow->isFreeze()
        ) {
            $builder
                ->add('freezeAfter', CheckboxType::class, [
                    'required' => false,
                    'label' => 'workflow.Freeze',
                    'help' => 'workflow.The associated element will be freezed',
                ]);
        }

        $builder
            ->add('finalizeAfter', CheckboxType::class, [
                'required' => false,
                'label' => 'workflow.Finalize',
                'help' => 'workflow.The workflow will be finalized',
            ])
            ->add('comment', ChillTextareaType::class, [
                'required' => false,
                'label' => 'Comment',
            ]);
    }

    public function configureOptions(OptionsResolver $resolver)
    {
        $resolver
            ->setDefined('class', EntityWorkflowStep::class)
            ->setRequired('transition')
            ->setAllowedTypes('transition', 'bool')
            ->setRequired('entity_workflow')
            ->setAllowedTypes('entity_workflow', EntityWorkflow::class);
    }
}
